@@ -1,4 +1,4 @@
-# Copyright (c) 2006-2017  Andrey Golovigin
+# Copyright (c) 2006-2021  Andrey Golovizin
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -19,15 +19,19 @@
 # TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 # SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+from __future__ import absolute_import, unicode_literals
+
 import re
 
+from pybtex.bibtex.exceptions import BibTeXError
 from pybtex.utils import pairwise
 
 whitespace_re = re.compile(r'(\s)')
 purify_special_char_re = re.compile(r'^\\[A-Za-z]+')
 
+
 def wrap(string, width=79, subsequent_indent='  '):
-    """
+    r"""
     Wrap long string into multiple lines by inserting line breaks.
 
     The string is broken at whitespace characters so that each line is as long
@@ -44,25 +48,25 @@ def wrap(string, width=79, subsequent_indent='  '):
     (3 characters by default), so that each line contains at least one
     non-whitespace character after the indent.
 
-    >>> print wrap('', width=3)
+    >>> print(wrap('', width=3))
     <BLANKLINE>
-    >>> print wrap('0123456789 12345', width=10)
+    >>> print(wrap('0123456789 12345', width=10))
     0123456789
       12345
-    >>> print wrap('01234 6789 12345', width=10)
+    >>> print(wrap('01234 6789 12345', width=10))
     01234 6789
       12345
-    >>> print wrap('01234 6789 12345', width=11)
+    >>> print(wrap('01234 6789 12345', width=11))
     01234 6789
       12345
-    >>> print wrap('01234 6789 12345', width=9)
+    >>> print(wrap('01234 6789 12345', width=9))
     01234
       6789
       12345
-    >>> print wrap(' a b c', width=3)
+    >>> print(wrap(' a b c', width=3))
      a b
       c
-    >>> print wrap('aa bb c', width=3)
+    >>> print(wrap('aa bb c', width=3))
     aa bb
       c
 
@@ -90,7 +94,10 @@ def wrap(string, width=79, subsequent_indent='  '):
 
 
 class BibTeXString(object):
-    def __init__(self, chars, level=0):
+    def __init__(self, chars, level=0, max_level=100):
+        if level > max_level:
+            raise BibTeXError('too many nested braces')
+
         self.level = level
         self.is_closed = False
         self.contents = list(self.find_closing_brace(iter(chars)))
@@ -132,34 +139,34 @@ class BibTeXString(object):
         if close is not None and self.level > 0 and self.is_closed:
             yield close(self)
 
-    def __unicode__(self):
+    def __str__(self):
         return ''.join(self.traverse(open=lambda string: '{', close=lambda string: '}'))
 
     def inner_string(self):
-        return ''.join(unicode(child) for child in self.contents)
+        return ''.join(str(child) for child in self.contents)
 
 
 def change_case(string, mode):
     r"""
-    >>> print change_case('aBcD', 'l')
+    >>> print(change_case('aBcD', 'l'))
     abcd
-    >>> print change_case('aBcD', 'u')
+    >>> print(change_case('aBcD', 'u'))
     ABCD
-    >>> print change_case('ABcD', 't')
+    >>> print(change_case('ABcD', 't'))
     Abcd
-    >>> print change_case(r'The {\TeX book \noop}', 'u')
+    >>> print(change_case(r'The {\TeX book \noop}', 'u'))
     THE {\TeX BOOK \noop}
-    >>> print change_case(r'And Now: BOOO!!!', 't')
+    >>> print(change_case(r'And Now: BOOO!!!', 't'))
     And now: Booo!!!
-    >>> print change_case(r'And {Now: BOOO!!!}', 't')
+    >>> print(change_case(r'And {Now: BOOO!!!}', 't'))
     And {Now: BOOO!!!}
-    >>> print change_case(r'And {Now: {BOOO}!!!}', 'l')
+    >>> print(change_case(r'And {Now: {BOOO}!!!}', 'l'))
     and {Now: {BOOO}!!!}
-    >>> print change_case(r'And {\Now: BOOO!!!}', 't')
+    >>> print(change_case(r'And {\Now: BOOO!!!}', 't'))
     And {\Now: booo!!!}
-    >>> print change_case(r'And {\Now: {BOOO}!!!}', 'l')
+    >>> print(change_case(r'And {\Now: {BOOO}!!!}', 'l'))
     and {\Now: {booo}!!!}
-    >>> print change_case(r'{\TeX\ and databases\Dash\TeX DBI}', 't')
+    >>> print(change_case(r'{\TeX\ and databases\Dash\TeX DBI}', 't'))
     {\TeX\ and databases\Dash\TeX DBI}
     """
 
@@ -206,27 +213,27 @@ def change_case(string, mode):
 
 
 def bibtex_substring(string, start, length):
-    """
+    r"""
     Return a substring of the given length, starting from the given position.
 
     start and length are 1-based. If start is < 0, it is counted from the end
     of the string. If start is 0, an empty string is returned.
 
-    >>> print bibtex_substring('abcdef', 1, 3)
+    >>> print(bibtex_substring('abcdef', 1, 3))
     abc
-    >>> print bibtex_substring('abcdef', 2, 3)
+    >>> print(bibtex_substring('abcdef', 2, 3))
     bcd
-    >>> print bibtex_substring('abcdef', 2, 1000)
+    >>> print(bibtex_substring('abcdef', 2, 1000))
     bcdef
-    >>> print bibtex_substring('abcdef', 0, 1000)
+    >>> print(bibtex_substring('abcdef', 0, 1000))
     <BLANKLINE>
-    >>> print bibtex_substring('abcdef', -1, 1)
+    >>> print(bibtex_substring('abcdef', -1, 1))
     f
-    >>> print bibtex_substring('abcdef', -1, 2)
+    >>> print(bibtex_substring('abcdef', -1, 2))
     ef
-    >>> print bibtex_substring('abcdef', -2, 3)
+    >>> print(bibtex_substring('abcdef', -2, 3))
     cde
-    >>> print bibtex_substring('abcdef', -2, 1000)
+    >>> print(bibtex_substring('abcdef', -2, 1000))
     abcde
     """
 
@@ -248,29 +255,29 @@ def bibtex_len(string):
     is a substring at brace level 1, if the first character after the opening
     brace is a backslash, like in "de la Vall{\'e}e Poussin".
 
-    >>> print bibtex_len(r"de la Vall{\'e}e Poussin")
+    >>> print(bibtex_len(r"de la Vall{\'e}e Poussin"))
     20
-    >>> print bibtex_len(r"de la Vall{e}e Poussin")
+    >>> print(bibtex_len(r"de la Vall{e}e Poussin"))
     20
-    >>> print bibtex_len(r"de la Vallee Poussin")
+    >>> print(bibtex_len(r"de la Vallee Poussin"))
     20
-    >>> print bibtex_len(r'\ABC 123')
+    >>> print(bibtex_len(r'\ABC 123'))
     8
-    >>> print bibtex_len(r'{\abc}')
+    >>> print(bibtex_len(r'{\abc}'))
     1
-    >>> print bibtex_len(r'{\abc')
+    >>> print(bibtex_len(r'{\abc'))
     1
-    >>> print bibtex_len(r'}\abc')
+    >>> print(bibtex_len(r'}\abc'))
     4
-    >>> print bibtex_len(r'\abc}')
+    >>> print(bibtex_len(r'\abc}'))
     4
-    >>> print bibtex_len(r'\abc{')
+    >>> print(bibtex_len(r'\abc{'))
     4
-    >>> print bibtex_len(r'level 0 {1 {2}}')
+    >>> print(bibtex_len(r'level 0 {1 {2}}'))
     11
-    >>> print bibtex_len(r'level 0 {\1 {2}}')
+    >>> print(bibtex_len(r'level 0 {\1 {2}}'))
     9
-    >>> print bibtex_len(r'level 0 {1 {\2}}')
+    >>> print(bibtex_len(r'level 0 {1 {\2}}'))
     12
     """
     length = 0
@@ -314,25 +321,25 @@ def bibtex_width(string):
 
 
 def bibtex_prefix(string, num_chars):
-    """Return the firxt num_char characters of the string.
+    r"""Return the firxt num_char characters of the string.
 
     Braces and "special characters" are ignored, as in bibtex_len.  If the
     resulting prefix ends at brace level > 0, missing closing braces are
     appended.
 
-    >>> print bibtex_prefix('abc', 1)
+    >>> print(bibtex_prefix('abc', 1))
     a
-    >>> print bibtex_prefix('abc', 5)
+    >>> print(bibtex_prefix('abc', 5))
     abc
-    >>> print bibtex_prefix('ab{c}d', 3)
+    >>> print(bibtex_prefix('ab{c}d', 3))
     ab{c}
-    >>> print bibtex_prefix('ab{cd}', 3)
+    >>> print(bibtex_prefix('ab{cd}', 3))
     ab{c}
-    >>> print bibtex_prefix('ab{cd', 3)
+    >>> print(bibtex_prefix('ab{cd', 3))
     ab{c}
-    >>> print bibtex_prefix(r'ab{\cd}', 3)
+    >>> print(bibtex_prefix(r'ab{\cd}', 3))
     ab{\cd}
-    >>> print bibtex_prefix(r'ab{\cd', 3)
+    >>> print(bibtex_prefix(r'ab{\cd', 3))
     ab{\cd}
 
     """
@@ -352,33 +359,33 @@ def bibtex_prefix(string, num_chars):
 def bibtex_purify(string):
     r"""Strip special characters from the string.
 
-    >>> print bibtex_purify('Abc 1234')
+    >>> print(bibtex_purify('Abc 1234'))
     Abc 1234
-    >>> print bibtex_purify('Abc  1234')
+    >>> print(bibtex_purify('Abc  1234'))
     Abc  1234
-    >>> print bibtex_purify('Abc-Def')
+    >>> print(bibtex_purify('Abc-Def'))
     Abc Def
-    >>> print bibtex_purify('Abc-~-Def')
+    >>> print(bibtex_purify('Abc-~-Def'))
     Abc   Def
-    >>> print bibtex_purify('{XXX YYY}')
+    >>> print(bibtex_purify('{XXX YYY}'))
     XXX YYY
-    >>> print bibtex_purify('{XXX {YYY}}')
+    >>> print(bibtex_purify('{XXX {YYY}}'))
     XXX YYY
-    >>> print bibtex_purify(r'XXX {\YYY} XXX')
+    >>> print(bibtex_purify(r'XXX {\YYY} XXX'))
     XXX  XXX
-    >>> print bibtex_purify(r'{XXX {\YYY} XXX}')
+    >>> print(bibtex_purify(r'{XXX {\YYY} XXX}'))
     XXX YYY XXX
-    >>> print bibtex_purify(r'\\abc def')
+    >>> print(bibtex_purify(r'\\abc def'))
     abc def
-    >>> print bibtex_purify('a@#$@#$b@#$@#$c')
+    >>> print(bibtex_purify('a@#$@#$b@#$@#$c'))
     abc
-    >>> print bibtex_purify(r'{\noopsort{1973b}}1973')
+    >>> print(bibtex_purify(r'{\noopsort{1973b}}1973'))
     1973b1973
-    >>> print bibtex_purify(r'{sort{1973b}}1973')
+    >>> print(bibtex_purify(r'{sort{1973b}}1973'))
     sort1973b1973
-    >>> print bibtex_purify(r'{sort{\abc1973b}}1973')
+    >>> print(bibtex_purify(r'{sort{\abc1973b}}1973'))
     sortabc1973b1973
-    >>> print bibtex_purify(r'{\noopsort{1973a}}{\switchargs{--90}{1968}}')
+    >>> print(bibtex_purify(r'{\noopsort{1973a}}{\switchargs{--90}{1968}}'))
     1973a901968
     """
 
@@ -412,7 +419,7 @@ def scan_bibtex_string(string):
 
 
 def split_name_list(string):
-    """
+    r"""
     Split a list of names, separated by ' and '.
 
     >>> split_name_list('Johnson and Peterson')
@@ -435,63 +442,109 @@ def split_name_list(string):
     return split_tex_string(string, ' [Aa][Nn][Dd] ')
 
 
+def _find_closing_brace(string):
+    r"""
+    >>> _find_closing_brace('')
+    ('', '')
+    >>> _find_closing_brace('no braces')
+    ('no braces', '')
+    >>> _find_closing_brace('brace at the end}')
+    ('brace at the end}', '')
+    >>> _find_closing_brace('two closing braces}}')
+    ('two closing braces}', '}')
+    >>> _find_closing_brace('two closing} braces} and some text')
+    ('two closing}', ' braces} and some text')
+    >>> _find_closing_brace('more {nested{}}{braces}} and the rest}')
+    ('more {nested{}}{braces}}', ' and the rest}')
+    """
+    up_to_brace = []
+    brace_level = 1
+    while brace_level >= 1:
+        next_brace = BRACE_RE.search(string)
+        if not next_brace:
+            break
+
+        up_to_brace.append(string[:next_brace.end()])
+        string = string[next_brace.end():]
+
+        if next_brace.group() == '{':
+            brace_level += 1
+        elif next_brace.group() == '}':
+            brace_level -= 1
+        else:
+            raise ValueError(next_brace.group())
+
+    if not up_to_brace:
+        up_to_brace, string = [string], ''
+    return ''.join(up_to_brace), string
+
+
+# "\ " is a "control space" in TeX, i. e. "a space that is not to be ignored"
+#     -- The TeXbook, Chapter 3: Controlling TeX, p 8
+# ~ is a space character, according to BibTeX
+# \~ is not a space character
+BIBTEX_SPACE_RE = re.compile(r'(?:\\ |\s|(?<!\\)~)+')
+BRACE_RE = re.compile(r'{|}')
+
+
 def split_tex_string(string, sep=None, strip=True, filter_empty=False):
     r"""Split a string using the given separator (regexp).
 
     Everything at brace level > 0 is ignored.
-    Separators at the edges of the string are ignored.
 
     >>> split_tex_string('')
     []
     >>> split_tex_string('     ')
     []
-    >>> split_tex_string('   ', ' ', strip=False, filter_empty=False)
-    [' ', ' ']
     >>> split_tex_string('.a.b.c.', r'\.')
-    ['.a', 'b', 'c.']
+    ['', 'a', 'b', 'c', '']
     >>> split_tex_string('.a.b.c.{d.}.', r'\.')
-    ['.a', 'b', 'c', '{d.}.']
+    ['', 'a', 'b', 'c', '{d.}', '']
     >>> split_tex_string('Matsui      Fuuka')
     ['Matsui', 'Fuuka']
     >>> split_tex_string('{Matsui      Fuuka}')
     ['{Matsui      Fuuka}']
     >>> split_tex_string(r'Matsui\ Fuuka')
     ['Matsui', 'Fuuka']
-    >>> split_tex_string('{Matsui\ Fuuka}')
+    >>> split_tex_string(r'{Matsui\ Fuuka}')
     ['{Matsui\\ Fuuka}']
     >>> split_tex_string('a')
     ['a']
     >>> split_tex_string('on a')
     ['on', 'a']
+    >>> split_tex_string(r'Qui\~{n}onero-Candela, J.')
+    ['Qui\\~{n}onero-Candela,', 'J.']
     """
 
     if sep is None:
-        # "\ " is a "control space" in TeX,
-        # i. e. "a space that is not to be ignored"
-        # The TeXbook, Chapter 3: Controlling TeX, p 8
-        sep = r'(\\ |[\s~])+'
+        sep = BIBTEX_SPACE_RE
         filter_empty = True
 
-    sep_re = re.compile(sep)
-    brace_level = 0
-    name_start = 0
+    sep = re.compile(sep)
+
     result = []
-    string_len = len(string)
-    pos = 0
-    for pos, char in enumerate(string):
-        if char == '{':
-            brace_level += 1
-        elif char == '}':
-            brace_level -= 1
-        elif brace_level == 0 and pos > 0:
-            match = sep_re.match(string[pos:])
-            if match:
-                sep_len = len(match.group())
-                if pos + sep_len < string_len:
-                    result.append(string[name_start:pos])
-                    name_start = pos + sep_len
-    if name_start < string_len:
-        result.append(string[name_start:])
+    word_parts = []
+
+    while True:
+        head, brace, string = string.partition('{')
+
+        if head:
+            head_parts = sep.split(head)
+            for word in head_parts[:-1]:
+                result.append(''.join(word_parts + [word]))
+                word_parts = []
+            word_parts.append(head_parts[-1])
+
+        if brace:
+            word_parts.append(brace)
+            up_to_closing_brace, string = _find_closing_brace(string)
+            word_parts.append(up_to_closing_brace)
+        else:
+            break
+
+    if word_parts:
+        result.append(''.join(word_parts))
+
     if strip:
         result = [part.strip() for part in result]
     if filter_empty:
@@ -500,42 +553,42 @@ def split_tex_string(string, sep=None, strip=True, filter_empty=False):
 
 
 def bibtex_first_letter(string):
-    """ Return the first letter or special character of the string.
+    r""" Return the first letter or special character of the string.
 
-    >>> print bibtex_first_letter('Andrew Blake')
+    >>> print(bibtex_first_letter('Andrew Blake'))
     A
-    >>> print bibtex_first_letter('{Andrew} Blake')
+    >>> print(bibtex_first_letter('{Andrew} Blake'))
     A
-    >>> print bibtex_first_letter('1Andrew')
+    >>> print(bibtex_first_letter('1Andrew'))
     A
-    >>> print bibtex_first_letter('{\TeX} markup')
+    >>> print(bibtex_first_letter(r'{\TeX} markup'))
     {\TeX}
-    >>> print bibtex_first_letter('')
+    >>> print(bibtex_first_letter(''))
     <BLANKLINE>
-    >>> print bibtex_first_letter('123 123 123 {}')
+    >>> print(bibtex_first_letter('123 123 123 {}'))
     <BLANKLINE>
-    >>> print bibtex_first_letter('\LaTeX Project Team')
+    >>> print(bibtex_first_letter(r'\LaTeX Project Team'))
     L
 
     """
 
     for char in BibTeXString(string):
         if char.startswith('\\') and char != '\\':
-            return u'{{{0}}}'.format(char)
+            return '{{{0}}}'.format(char)
         elif char.isalpha():
             return char
     return ''
 
 
 def bibtex_abbreviate(string, delimiter=None, separator='-'):
-    """
+    r"""
     Abbreviate string.
 
-    >>> print bibtex_abbreviate('Andrew Blake')
+    >>> print(bibtex_abbreviate('Andrew Blake'))
     A
-    >>> print bibtex_abbreviate('Jean-Pierre')
+    >>> print(bibtex_abbreviate('Jean-Pierre'))
     J.-P
-    >>> print bibtex_abbreviate('Jean--Pierre')
+    >>> print(bibtex_abbreviate('Jean--Pierre'))
     J.-P
     
     """
