@@ -1,5 +1,6 @@
 # vim: fileencoding=utf-8
 # Copyright (c) 2006-2021  Andrey Golovizin
+# Copyright (c)      2022  Julian Rüth
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -23,13 +24,13 @@
 from __future__ import unicode_literals
 from __future__ import print_function
 import re
+from typing import Iterable, Tuple
 
 try:
     from collections.abc import Mapping
 except ImportError:
     from collections import Mapping
 
-import six
 import textwrap
 
 from pybtex.exceptions import PybtexError
@@ -40,7 +41,6 @@ from pybtex.utils import (
 from pybtex.richtext import Text
 from pybtex.bibtex.utils import split_tex_string, scan_bibtex_string
 from pybtex.errors import report_error
-from pybtex.py3compat import fix_unicode_literals_in_doctest, python_2_unicode_compatible
 from pybtex.plugin import find_plugin
 
 
@@ -135,6 +135,23 @@ class BibliographyData(object):
     def add_to_preamble(self, *values):
         self._preamble.extend(values)
 
+    @property
+    def preamble_list(self):
+        r'''
+        LaTeX preamble as list of strings
+
+        >>> bib_data = parse_string(r"""
+        ...     @PREAMBLE{"\newcommand{\noopsort}[1]{}"}
+        ...     @PREAMBLE{"\newcommand{\nooptilde}[1]{}"}
+        ... """, 'bibtex')
+        >>> print(bib_data.preamble_list)
+        ['\\newcommand{\\noopsort}[1]{}', '\\newcommand{\\nooptilde}[1]{}']
+
+        .. versionadded:: 0.19
+            Earlier versions used :py:meth:`.get_preamble()`, which is now deprecated.
+        '''
+        return self._preamble
+
     @property
     def preamble(self):
         r'''
@@ -176,7 +193,7 @@ class BibliographyData(object):
         if not self.want_entry(key):
             return
         if key in self.entries:
-            report_error(BibliographyDataError('repeated bibliograhpy entry: %s' % key))
+            report_error(BibliographyDataError('repeated bibliography entry: %s' % key))
             return
         entry.key = self.get_canonical_key(key)
         self.entries[entry.key] = entry
@@ -188,11 +205,10 @@ class BibliographyData(object):
             if self.wanted_entries is not None:
                 self.wanted_entries.add(crossref)
 
-    def add_entries(self, entries):
+    def add_entries(self, entries: Iterable[Tuple[str, "Entry"]]) -> None:
         for key, entry in entries:
             self.add_entry(key, entry)
 
-    @fix_unicode_literals_in_doctest
     def _get_crossreferenced_citations(self, citations, min_crossrefs):
         r"""
         Get cititations not cited explicitly but referenced by other citations.
@@ -205,9 +221,9 @@ class BibliographyData(object):
         >>> list(data._get_crossreferenced_citations([], min_crossrefs=1))
         []
         >>> list(data._get_crossreferenced_citations(['main_article'], min_crossrefs=1))
-        [u'xrefd_arcicle']
+        ['xrefd_arcicle']
         >>> list(data._get_crossreferenced_citations(['Main_article'], min_crossrefs=1))
-        [u'xrefd_arcicle']
+        ['xrefd_arcicle']
         >>> list(data._get_crossreferenced_citations(['main_article'], min_crossrefs=2))
         []
         >>> list(data._get_crossreferenced_citations(['xrefd_arcicle'], min_crossrefs=1))
@@ -217,9 +233,9 @@ class BibliographyData(object):
         >>> list(data2._get_crossreferenced_citations([], min_crossrefs=1))
         []
         >>> list(data2._get_crossreferenced_citations(['main_article'], min_crossrefs=1))
-        [u'xrefd_arcicle']
+        ['xrefd_arcicle']
         >>> list(data2._get_crossreferenced_citations(['Main_article'], min_crossrefs=1))
-        [u'xrefd_arcicle']
+        ['xrefd_arcicle']
         >>> list(data2._get_crossreferenced_citations(['main_article'], min_crossrefs=2))
         []
         >>> list(data2._get_crossreferenced_citations(['xrefd_arcicle'], min_crossrefs=1))
@@ -254,7 +270,6 @@ class BibliographyData(object):
                 citation_set.add(canonical_crossref)
                 yield canonical_crossref
 
-    @fix_unicode_literals_in_doctest
     def _expand_wildcard_citations(self, citations):
         r"""
         Expand wildcard citations (\citation{*} in .aux file).
@@ -269,15 +284,15 @@ class BibliographyData(object):
         >>> list(data._expand_wildcard_citations([]))
         []
         >>> list(data._expand_wildcard_citations(['*']))
-        [u'uno', u'dos', u'tres', u'cuatro']
+        ['uno', 'dos', 'tres', 'cuatro']
         >>> list(data._expand_wildcard_citations(['uno', '*']))
-        [u'uno', u'dos', u'tres', u'cuatro']
+        ['uno', 'dos', 'tres', 'cuatro']
         >>> list(data._expand_wildcard_citations(['dos', '*']))
-        [u'dos', u'uno', u'tres', u'cuatro']
+        ['dos', 'uno', 'tres', 'cuatro']
         >>> list(data._expand_wildcard_citations(['*', 'uno']))
-        [u'uno', u'dos', u'tres', u'cuatro']
+        ['uno', 'dos', 'tres', 'cuatro']
         >>> list(data._expand_wildcard_citations(['*', 'DOS']))
-        [u'uno', u'dos', u'tres', u'cuatro']
+        ['uno', 'dos', 'tres', 'cuatro']
 
         """
 
@@ -341,16 +356,15 @@ class BibliographyData(object):
 
         .. versionadded:: 0.19
         """
-        if isinstance(file, six.string_types):
+        if isinstance(file, str):
             filename = file
         else:
             filename = getattr(file, 'name', None)
         writer = find_plugin('pybtex.database.output', bib_format, filename=filename)(**kwargs)
         return writer.write_file(self, file)
 
-    @fix_unicode_literals_in_doctest
     def lower(self):
-        u'''
+        '''
         Return another :py:class:`.BibliographyData` with all identifiers converted to lowercase.
 
         >>> data = parse_string("""
@@ -365,17 +379,17 @@ class BibliographyData(object):
         ... """, 'bibtex')
         >>> data_lower = data.lower()
         >>> list(data_lower.entries.keys())
-        [u'obrazy', u'elegie']
+        ['obrazy', 'elegie']
         >>> for entry in data_lower.entries.values():
         ...     entry.key
         ...     list(entry.persons.keys())
         ...     list(entry.fields.keys())
-        u'obrazy'
-        [u'author']
-        [u'title']
-        u'elegie'
-        [u'author']
-        [u'title']
+        'obrazy'
+        ['author']
+        ['title']
+        'elegie'
+        ['author']
+        ['title']
 
         '''
 
@@ -447,22 +461,31 @@ class Entry(object):
         )
 
     def __repr__(self):
-        # represent the fields as a list of tuples for simplicity
-        repr_fields = repr(list(self.fields.items()))
-        keys = self.fields.keys()
+        r"""
+        Return a string representation of this entry.
 
-        for key in keys:
-            ind = repr_fields.index(key) - 2  # find first instance
-            repr_fields = repr_fields[:ind] + "\n" + repr_fields[ind:]
+        >>> from pybtex.database import Entry
+        >>> Entry("article")
+        Entry('article')
 
-        repr_fields = indent(repr_fields, prefix="    ")
-        repr_fields = repr_fields[4:]  # drop 1st indent
+        >>> Entry("article", fields={"title": "An article about journals", "journal": "Royal Society of Journals"}, persons={"editor": Person("Donald E. Knuth")})
+        Entry('article',
+          fields=[
+            ('title', 'An article about journals'),
+            ('journal', 'Royal Society of Journals')],
+          persons={'editor': Person('Knuth, Donald E.')})
 
-        return (
-            "Entry({0},\n"
-            "  fields={1},\n"
-            "  persons={2})".format(repr(self.type), repr_fields, repr(self.persons))
-        )
+        """
+        parameters = [repr(self.type)]
+
+        if self.fields:
+            parameters.append("fields=[\n    {}]".format(",\n    ".join(
+                ["({!r}, {!r})".format(key, value) for (key, value) in self.fields.items()])))
+
+        if self.persons:
+            parameters.append("persons={!r}".format(dict(self.persons)))
+
+        return "Entry({})".format(",\n  ".join(parameters))
 
     def add_person(self, person, role):
         self.persons.setdefault(role, []).append(person)
@@ -476,7 +499,7 @@ class Entry(object):
 
     def _find_person_field(self, role):
         persons = self.persons[role]
-        return ' and '.join(six.text_type(person) for person in persons)
+        return ' and '.join(str(person) for person in persons)
 
     def _find_crossref_field(self, name, bib_data):
         if bib_data is None or 'crossref' not in self.fields:
@@ -534,18 +557,16 @@ class Entry(object):
         return bibdata.entries[key]
 
 
-@python_2_unicode_compatible
-@fix_unicode_literals_in_doctest
 class Person(object):
     """A person or some other person-like entity.
 
     >>> knuth = Person('Donald E. Knuth')
     >>> knuth.first_names
-    [u'Donald']
+    ['Donald']
     >>> knuth.middle_names
-    [u'E.']
+    ['E.']
     >>> knuth.last_names
-    [u'Knuth']
+    ['Knuth']
 
     """
 
@@ -625,7 +646,6 @@ class Person(object):
         self.lineage_names.extend(split_tex_string(lineage))
 
     @property
-    @fix_unicode_literals_in_doctest
     def bibtex_first_names(self):
         """A list of first and middle names together.
         (BibTeX treats all middle names as first.)
@@ -636,51 +656,50 @@ class Person(object):
 
         >>> knuth = Person('Donald E. Knuth')
         >>> knuth.bibtex_first_names
-        [u'Donald', u'E.']
+        ['Donald', 'E.']
         """
         return self.first_names + self.middle_names
 
-    @fix_unicode_literals_in_doctest
     def _parse_string(self, name):
         """Extract various parts of the name from a string.
 
         >>> p = Person('Avinash K. Dixit')
         >>> print(p.first_names)
-        [u'Avinash']
+        ['Avinash']
         >>> print(p.middle_names)
-        [u'K.']
+        ['K.']
         >>> print(p.prelast_names)
         []
         >>> print(p.last_names)
-        [u'Dixit']
+        ['Dixit']
         >>> print(p.lineage_names)
         []
-        >>> print(six.text_type(p))
+        >>> print(str(p))
         Dixit, Avinash K.
-        >>> p == Person(six.text_type(p))
+        >>> p == Person(str(p))
         True
         >>> p = Person('Dixit, Jr, Avinash K. ')
         >>> print(p.first_names)
-        [u'Avinash']
+        ['Avinash']
         >>> print(p.middle_names)
-        [u'K.']
+        ['K.']
         >>> print(p.prelast_names)
         []
         >>> print(p.last_names)
-        [u'Dixit']
+        ['Dixit']
         >>> print(p.lineage_names)
-        [u'Jr']
-        >>> print(six.text_type(p))
+        ['Jr']
+        >>> print(str(p))
         Dixit, Jr, Avinash K.
-        >>> p == Person(six.text_type(p))
+        >>> p == Person(str(p))
         True
 
         >>> p = Person('abc')
         >>> print(p.first_names, p.middle_names, p.prelast_names, p.last_names, p.lineage_names)
-        [] [] [] [u'abc'] []
+        [] [] [] ['abc'] []
         >>> p = Person('Viktorov, Michail~Markovitch')
         >>> print(p.first_names, p.middle_names, p.prelast_names, p.last_names, p.lineage_names)
-        [u'Michail'] [u'Markovitch'] [] [u'Viktorov'] []
+        ['Michail'] ['Markovitch'] [] ['Viktorov'] []
         """
         def process_first_middle(parts):
             try:
@@ -788,21 +807,20 @@ class Person(object):
         return ', '.join(part for part in (von_last, jr, first) if part)
 
     def __repr__(self):
-        return 'Person({0})'.format(repr(six.text_type(self)))
+        return 'Person({0})'.format(repr(str(self)))
 
     def get_part_as_text(self, type):
         names = getattr(self, type + '_names')
         return ' '.join(names)
 
-    @fix_unicode_literals_in_doctest
     def get_part(self, type, abbr=False):
         """Get a list of name parts by `type`.
 
         >>> knuth = Person('Donald E. Knuth')
         >>> knuth.get_part('first')
-        [u'Donald']
+        ['Donald']
         >>> knuth.get_part('last')
-        [u'Knuth']
+        ['Knuth']
         """
 
         names = getattr(self, type + '_names')
@@ -919,7 +937,7 @@ def parse_file(file, bib_format=None, **kwargs):
     .. versionadded:: 0.19
     """
 
-    if isinstance(file, six.string_types):
+    if isinstance(file, str):
         filename = file
     else:
         filename = getattr(file, 'name', None)
