@@ -1,4 +1,4 @@
-# Copyright (c) 2006-2019  Andrey Golovigin
+# Copyright (c) 2006-2021  Andrey Golovizin
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -23,8 +23,6 @@ from __future__ import absolute_import, unicode_literals
 
 import sys
 
-import six
-
 
 class PybtexError(Exception):
     def __init__(self, message, filename=None):
@@ -37,14 +35,14 @@ class PybtexError(Exception):
 
     def get_filename(self):
         """Return filename, if relevant."""
-        if self.filename is None or isinstance(self.filename, six.text_type):
+        if self.filename is None or isinstance(self.filename, str):
             return self.filename
         else:
             from .io import _decode_filename
             return _decode_filename(self.filename, errors='replace')
 
     def __eq__(self, other):
-        return six.text_type(self) == six.text_type(other)
+        return str(self) == str(other)
 
     def __hash__(self):
-        return hash(six.text_type(self))
+        return hash(str(self))
